// ----LCD----
// SCL - A5
// SDA - A4
// GND - GND
// VCC - 5V
#include <LiquidCrystal_I2C.h>
LiquidCrystal_I2C lcd(0x27, 2, 1, 0, 4, 5, 6, 7, 3, POSITIVE);

#define Trig 2 
#define Echo 3 
#define ldrPin A3
#define buzzerPin 6
#define buttonPin 7
#define ledPin 5 //GREEN
#define ledrPin 4 //RED

#define DD 3 //距离大于该值判定为前倾
#define LL 8 //当前光强与初始坐下状态差值大于该值判定为站立状态
#define trest 20  //坐着时长大于该值判定为久坐
#define freq1 300  //身体前倾蜂鸣器频率
#define freq2 250  //久坐提醒蜂鸣器频率
#define delaytime 200 //刷新频率

int ButtonState = LOW;
int ldrv0=0;
float d0=0;
int sit=0;
long int tstart;
//int num=0;

void setup() {
  //Serial.begin(9600);
  pinMode(ldrPin,INPUT);
  pinMode(ledPin,OUTPUT);
  pinMode(ledrPin,OUTPUT);
  pinMode(buzzerPin,OUTPUT);
  pinMode(buttonPin,INPUT);
  pinMode(Trig, OUTPUT);
  pinMode(Echo, INPUT);
  lcd.begin(16, 2);
  lcd.print("Sit correctly");
  lcd.setCursor(0,1);
  lcd.print("Press to init");
  while(ButtonState==LOW){
    ButtonState = digitalRead(buttonPin);
    if(ButtonState==HIGH){
      for(int i=0;i<5;i++){
        d0+=distance();
        ldrv0+=analogRead(ldrPin);
        delay(100);
      }
      d0=d0/5;
      ldrv0=ldrv0/5;
      lcd.clear();
      lcd.print(d0);
      lcd.setCursor(0,1);
      lcd.print(ldrv0);
      sit=1;
      tstart=millis()/1000;
      delay(500);
      break;
    }
  }
}
void buzz(int freq){
  tone(buzzerPin,freq);
}

void nobuzz(){
  noTone(buzzerPin);
}

void ldr(){
  int ldrval=analogRead(ldrPin);
  //Serial.print("ldrValue=");
  //Serial.println(ldrval);
  //Serial.print("num=");
  //Serial.println(num);
  if(ldrval-ldrv0>LL){
    digitalWrite(ledPin,LOW);
    sit=0;
  }
  else{
    digitalWrite(ledPin,HIGH);
    sit=1;
  } 
  lcd.clear();
  lcd.setCursor(0,0);
  if(sit)lcd.print("state:sitting");
  else lcd.print("state:standing");
}

float distance(){
  float d; 
  float temp; 
  //给Trig发送一个低高低的短时间脉冲,触发测距
  digitalWrite(Trig, LOW);
  delayMicroseconds(2);
  digitalWrite(Trig,HIGH);
  delayMicroseconds(10);
  digitalWrite(Trig, LOW);
  
  temp = float(pulseIn(Echo, HIGH)); 
  d = (temp * 17 )/1000; //把回波时间换算为距离
  
  //Serial.print("Echo =");
  //Serial.print(temp);//串口输出等待时间的原始数据
  //Serial.print("Distance = ");
  //Serial.print(d);
  //Serial.println("cm");

  return d;
}

void loop() {
  ldr(); //坐着时亮绿灯
  if(sit){
    if(distance()-d0>DD){ //前倾
      buzz(freq1);
      digitalWrite(ledrPin,HIGH);
      lcd.clear();
      lcd.print("Bad posture!");
      //num++;
    }
    else{
      nobuzz();
      digitalWrite(ledrPin,LOW);
    }
    if(millis()/1000-tstart>trest){ //久坐提醒
      buzz(freq2);
      digitalWrite(ledrPin,HIGH);
      lcd.clear();
      lcd.print("Please rest!");
    }
    lcd.setCursor(0,1);
    lcd.print("Sit time:");
    lcd.setCursor(10,1);
    lcd.print(millis()/1000-tstart);
  }
  else{
    nobuzz();
    digitalWrite(ledrPin,LOW);
    /*
    lcd.setCursor(0,1);
    if(num==0)lcd.print("GOOD POSTURE!");
    else {
      lcd.print("BAD POSTURE:");
      lcd.setCursor(13,1);
      lcd.print(num*delaytime/1000);
    }*/
    tstart=millis()/1000;
  }
  delay(delaytime);
} 
